/************************************************************************/
/*																		*/
/*	DACSPI1.cpp		--		Definition for DACSPI1 library 	    		*/
/*																		*/
/************************************************************************/
/*	Author:		Cristian Fatu											*/
/*	Copyright 2012, Digilent Inc.										*/
/************************************************************************/
/*  File Description:													*/
/*		This file defines functions for DACSPI1							*/
/*																		*/
/************************************************************************/
/*  Revision History:													*/
/*																		*/
/*	01/16/2012(CristianF): created										*/
/*  06/19/2012(MichaelK) : Made floating point calculations conditional */
/*                         compilatioin                                 */
/*																		*/
/************************************************************************/


/* ------------------------------------------------------------ */
/*				Include File Definitions						*/
/* ------------------------------------------------------------ */
#include "DACSPI1.h"
#include <Dspi.h>


/* ------------------------------------------------------------ */
/*				Procedure Definitions							*/
/* ------------------------------------------------------------ */


/* ------------------------------------------------------------ */
/*        DACSPI1::DACSPI1
**
**        Synopsis:
**				
**        Parameters:
**
**
**
**        Return Values:
**                void 
**
**        Errors:
**
**
**        Description:
**			Class constructor. Performs variables initialization tasks
**
**
*/
DACSPI1::DACSPI1()
{
	pdspi = NULL;
}

/* ------------------------------------------------------------ */
/*        DACSPI1::WriteIntegerValue
**
**        Synopsis:
**				WriteIntegerValue(0x00FF);
**        Parameters:
**				- uint8_t bIntegerValue - the 8 bits value to be written to DA converter 
**
**        Return Value:
**                uint8_t 
**					- DACSPI1_ERR_SUCCESS (0) 				- The action completed successfully 
**					- DACSPI1_ERR_VAL_OUT_OF_RANGE	(1) 	- The value is not within the 0 - 0xFF range
**
**        Errors:
**			If module is not initialized (using begin), the function does nothing. Also, see return value.
**
**        Description:
**			If the value is inside the allowed range (0 - 0xFF), this function writes the 12 bits value to the DA converter, by writing 16 bits to SPI and returns the DACSPI1_ERR_SUCCESS status message.
**			If the value is outside the allowed range, the function does nothing and returns the DACSPI1_ERR_VAL_OUT_OF_RANGE message.
**
**
*/
uint8_t DACSPI1::WriteIntegerValue(uint8_t bIntegerValue)
{
	uint8_t bResult = 0;
	uint8_t *pbValue = (uint8_t *)&bIntegerValue;
	if(bIntegerValue < 0 || bIntegerValue >= (1 << DACSPI1_NO_BITS))
	{
		bResult = DACSPI1_ERR_VAL_OUT_OF_RANGE;
	}
	else
	{
		bResult = DACSPI1_ERR_SUCCESS;
		if(pdspi != NULL)
		{
			// make SS active
			digitalWrite(m_SSPin, LOW);		
			
			// write to SPI
			pdspi->transfer(DACSPI1_CTRL_BYTE); // high byte, the control bits (selecting DAC A)
			pdspi->transfer(bIntegerValue); // low byte

			// make SS inactive
			digitalWrite(m_SSPin, HIGH);
		}
	}
	return bResult;
}

/* ------------------------------------------------------------ */
/*        DACSPI1::WritePhysicalValue
**
**        Synopsis:
**				 myDACSPI1.WritePhysicalValue(dValue);
**        Parameters:
**				- float dPhysicalValue - the physical value that must be reconstructed at the output of the DA converter
**				- float dReference - the value corresponding to the maximum converter value (reference voltage). If this parameter is not provided, it has a default value of 3.3.
**									
**
**        Return Values:
**                uint8_t 
**					- DACSPI2_ERR_SUCCESS (0) 				- The action completed successfully 
**					- DACSPI2_ERR_VAL_OUT_OF_RANGE	(1) 	- The physical value is not inside accepted range
**
**        Errors:
**			If module is not initialized (using begin), the function does nothing. Also, see return value.
**
**        Description:
**			The function computes the integer value corresponding to the physical value by considering the reference value as the one corresponding to the maximum integer value (0xFF).
**			If the integer value is within the accepted range (0 - 0xFF), this function writes the 12- bit value to the DA converter, by writing 8 bits to SPI, and returns the DACSPI1_ERR_SUCCESS message.
**			If the integer value is outside the allowed range, the function does nothing and returns the DACSPI1_ERR_VAL_OUT_OF_RANGE message.
**			If the dReference function argument is missing, 3.3 value is used as reference value.
**
*/
#ifdef DA1_FLOATING_POINT

uint8_t DACSPI1::WritePhysicalValue(float dPhysicalValue, float dReference)
{
	uint8_t wIntegerValue = dPhysicalValue * (float)(1<<DACSPI1_NO_BITS) / dReference;
	return WriteIntegerValue(wIntegerValue);
}

#endif

/* ------------------------------------------------------------ */
/*        DACSPI1::begin
**
**        Synopsis:
**				myDACSPI1.begin(PAR_ACCESS_SPI0);
**        Parameters:
**				uint8_t bAccessType	- the SPI interface where the Pmod is connected. It can be one of:
**					0	PAR_ACCESS_SPI0	- indicates SPI port 2, connetor JB
**					1	PAR_ACCESS_SPI1	- indicates SPI port 1, connector J1
**
**
**        Return Values:
**                void 
**
**        Errors:
**
**
**        Description:
**				This function initializes the specific SPI interface used, setting the SPI frequency to a default value of 1 MHz.
**
**
*/
void DACSPI1::begin(uint8_t bAccessType)
{
	if(bAccessType == PAR_ACCESS_SPI0)
	{
		pdspi = new DSPI0();
		m_SSPin = PIN_DSPI0_SS;	// default SS pin for SPI0
	}
	if(bAccessType == PAR_ACCESS_SPI1)
	{
		pdspi = new DSPI1();
		m_SSPin = PIN_DSPI1_SS;	// default SS pin for SPI1
	}
	if(pdspi != NULL)
	{
		pdspi->begin(m_SSPin);	// this defines SS pin as output, sets SS high
		pdspi->setMode(DSPI_MODE3);
		pdspi->setSpeed(DACSPI1_SPI_FREQ);
	}
}





